require 'spec_helper'

describe GitoFigure do

  
  it "should clone a git repository" do
    Git.should_receive(:clone).
      with('gitolite@next.git.tw.com:gitolite-admin', 'next.git.tw.com', :path => '/tmp/gitolites').
      and_return(repo = stub(:git))
    repo.stub(:pull); repo.stub(:push)
    
    GitoFigure.administer('next.git.tw.com', :git_user => 'gitolite')
  end

  it "should not clone a repository if one already exists" do
    Dir.stub(:exists?).and_return(true)
    Git.should_not_receive(:clone)
    Git.should_receive(:open).with('/tmp/gitolites/test.git.tw.com').and_return(repo = stub(:git))
    repo.stub(:pull); repo.stub(:push)
    
    GitoFigure.administer('test.git.tw.com', :git_user => 'gitolite')    
  end

  xit "should raise if a repository already exists in the path but points to another origin" do
  end

  context "should accept a block" do
    it "and yield to it with a GitoFigure instance" do
      block = lambda { |admin| @gitofigure = admin }
      
      GitoFigure.administer('test.git.tw.com', &block)

      @gitofigure.repo.repo.to_s.should == '/tmp/gitolites/test.git.tw.com/.git'
    end

    it "enveloping it by a pull/push with upstream" do
      block = lambda { |x| @called = true }
      Git.stub(:open).and_return(repo = stub(:git))

      repo.should_receive(:pull).ordered
      repo.should_receive(:push).ordered

      GitoFigure.administer('test.git.tw.com', &block)

      @called.should be_true
    end
  end

  context "without a block" do
    it "should return an instance" do
      gitofigure = GitoFigure.administer('test.git.tw.com')
      gitofigure.repo.repo.to_s.should == '/tmp/gitolites/test.git.tw.com/.git'
    end
  end
  
  it "should push a given key upstream" do
    git = Git.open(TEST_REPO_PATH)
    
    GitoFigure.administer('test.git.tw.com') do |admin|
      admin.authorize_keys(:alpha => "xxxyyyzzz= alpha")
    end

    git.log.first.diff_parent.stats[:files].key?('keydir/alpha.pub').should be_true
    git.log.first.message.should == 'Added alpha.pub'
  end

  it "should push a given keyfile.pub file upstream" do
    FileUtils.touch('/tmp/key.pub')
    git = Git.open(TEST_REPO_PATH)
    
    GitoFigure.administer('test.git.tw.com') do |admin|
      admin.authorize_keyfile('/tmp/key.pub')
    end

    git.log.first.diff_parent.stats[:files].key?('keydir/key.pub').should be_true
    git.log.first.message.should == 'Added key.pub'
  end

end


